//
// Simple Rover (SRover)
//

#include <stdio.h>
#include "gb_drivers.h"

#ifdef _WIN32
#include <conio.h> // for windows _kbhit and _getch
#endif

#define BOARD 0  // Which board we talk to 
#define LEFT  0  // channel for left track
#define RIGHT 2  // channel for right track

#define STEP_PLUS  +500
#define STEP_MINUS -500
#define STEP_ZERO  0

int main(int argc, char *argv[])
{ int running,key;
  
  // Open serial port to talk to Gertbot 
  if (!open_connection(0))
    return 1;
    
  // Setup the channels for stepper motors
  set_mode(BOARD,LEFT,GB_MODE_STEPG_OFF);
  set_mode(BOARD,RIGHT,GB_MODE_STEPG_OFF);
  freq_stepper(BOARD,RIGHT,100.0);
  freq_stepper(BOARD,LEFT,100.0);
  
    
  printf("Gertbot example program\n");
  printf("This program is for two stepper motors\n");
  printf("It assumes the board number is set to %d\n",BOARD);
  printf("The motors are connected to outputs 0/1 (A/B) and 2/3 (C/D)\n");
  printf("Use numeric keypad keys control\n");
  printf("Left  Both  Right\n");
  printf("     Forward     \n");
  printf("  7     8     9  \n");
  printf("                 \n");
  printf("  4    Stop   6  \n");
  printf("                 \n");
  printf("  1     2     3  \n");
  printf("     Reverse     \n");
  printf("Left  Both  Right\n");
  printf("Don't forget to set numlock on!!!!\n");
  printf("Use Q or q to quit\n");
  printf("\n");
  
  // Go into loop
  running = 1;
  while (running)
  {
    if (_kbhit())
    { // A key was pressed 
      key = _getch(); // get the key 
      switch (key) // what to do with it 
      {
      case 'q' : 
      case 'Q' : running = 0;
                 break;
      case '1' : move_stepper(BOARD,LEFT,STEP_PLUS); // Left backwards 
                 break;
      case '2' : move_stepper(BOARD,LEFT,STEP_PLUS); // Left backwards 
                 move_stepper(BOARD,RIGHT,STEP_PLUS); // Right backwards 
                 break;
      case '3' : move_stepper(BOARD,RIGHT,STEP_PLUS); // Right backwards 
                 break;
      case '4' : move_stepper(BOARD,LEFT,STEP_ZERO); // Left stop 
                 break;
      case '5' : move_stepper(BOARD,LEFT,STEP_ZERO); // Left stop 
                 move_stepper(BOARD,RIGHT,STEP_ZERO); // Right stop 
                 break;
      case '6' : move_stepper(BOARD,RIGHT,STEP_ZERO); // Right stop 
                 break;
      case '7' : move_stepper(BOARD,LEFT,STEP_MINUS); // Left forwards 
                 break;
      case '8' : move_stepper(BOARD,LEFT,STEP_MINUS); // Left forwards 
                 move_stepper(BOARD,RIGHT,STEP_MINUS); // Right forwards 
                 break;
      case '9' : move_stepper(BOARD,RIGHT,STEP_MINUS); // Right forwards 
                 break;
      } // switch 
    } // kbhit
  } // while running 
  
  // At exit send emergency stop
  // to remove power from all motors 
  emergency_stop();
#ifdef _WIN32
#else
  // restore Linux terminal input to normal
  system("stty echo cooked");
#endif    
  
} // main 

#ifdef _WIN32
#else

// emulate windows kbhit and getch routines
// for linux 

int init = 0;
int c=0;
int _kbhit()
{ if (init==0)
  { // set terminal to no buffer I/O and no echo
    system("stty raw -echo");
    init=1;
  }
  if (c)
    return 1;
  c = getchar();
  if (c==-1)
  { // -1 means no character
    c = 0;
    return 0;
  }
  return 1;
} // _kbhit

int _getch()
{ int k;
  if (c)
  { k=c;
    c = 0;
    return k;
  }
  c = getchar();
  return c;
} // _getch

#endif 
